/**
 * @file
 * @brief Υ饹
 * @author ()
 */

#if __GNUC__ < 3
# include <stream.h>
# include <math.h>
# include <stdlib.h>
#else
# include <iostream>
# include <iomanip>
# include <cmath>
# include <cstdlib>
# include <new>         // std::bad_alloc ꤲʤ褦ˤ뤿
# if defined DEBUG
#  include <limits>     // std::numeric_limits<double>::quiet_NaN()
# endif
using std::endl;
using std::cerr;
using std::istream;
using std::ostream;
#endif

#include "Mat.h"
#include "nr.h"
#include "nradd.h"

namespace Lab
{
    // 󥹥ȥ饯1
    Mat::Mat(int row, int col)
    {
        allocate(row, col);
#if __GNUC__ >= 3 && defined DEBUG
        // ˺ˤץȯŤ餤ư򤱤뤿ˡ
        // ͤ NaN (Not a number) ǽƤ
        // (Ψθǡץץåܥ
        // DEBUG ƤΤߡȤ)
        for (int i = 0; i < row; i++)
            for (int j = 0; j < col; j++)
                mat_[i].vec_[j] = std::numeric_limits<double>::quiet_NaN();
#endif
    }

    Mat::Mat(
        double a, double b, double c, double d, double e,
        double f, double g, double h, double i)
    {
        allocate(3, 3);
        mat_[0].vec_[0] = a;
        mat_[0].vec_[1] = b;
        mat_[0].vec_[2] = c;
        mat_[1].vec_[0] = d;
        mat_[1].vec_[1] = e;
        mat_[1].vec_[2] = f;
        mat_[2].vec_[0] = g;
        mat_[2].vec_[1] = h;
        mat_[2].vec_[2] = i;
    }

    // 󥹥ȥ饯2
    Mat::Mat(const double* d, int row, int col)
    {
        allocate(row, col);
        const double* ptr = d;
        for (int i = 0; i < row; i++)
            for (int j = 0; j < col; j++)
                mat_[i].vec_[j] = *ptr++;
    }

    // 󥹥ȥ饯3
    Mat::Mat(const VecC& c1, const VecC& c2, const VecC& c3)
    {
        allocate(c1.size(), 3);
        for (int i = 0; i < rows_; i++)
        {
            mat_[i].vec_[0] = c1[i];
            mat_[i].vec_[1] = c2[i];
            mat_[i].vec_[2] = c3[i];
        }
    }

    // ԡ󥹥ȥ饯
    Mat::Mat(const Mat& M)
    {
        rows_ = M.rows_;
        cols_ = M.cols_;
        mat_ = new VecR[rows_];
        for (int i = 0; i < rows_; i++)
        {
            mat_[i].renew(cols_);
            for (int j = 0; j < cols_; j++)
                mat_[i].vec_[j] = M[i].vec_[j];
        }
    }

    // ǥȥ饯
    Mat::~Mat()
    {
       delete [] mat_;
    }

    // γ
    void  Mat::allocate(int row, int col)
    {
        rows_ = row;
        cols_ = col;
#if __GNUC__ < 3 || defined(USE_EXCEPTION)
        // new ϥݼԻ std::bad_alloc 㳰Ф
        mat_ = new VecR[row];
#else
        // new ϥݼԻ 0 ֤
        mat_ = new(std::nothrow) VecR[row];
#endif
        if (mat_ == NULL)
        {
            cerr << "### Memory Alloc Error (Mat.allocate) ###\n";
            ::abort();
        }

        for (int i = 0; i < row; i++)
            mat_[i].renew(col);
    }

    // ǡΥå
    void  Mat::clear(double data)
    {
       for (int i = 0; i < rows_; i++)
          for (int j = 0; j < cols_; j++)
             mat_[i].vec_[j] = data;
    }

    // ɽؿ(ǽ)
    void  Mat::print(ostream& s, const char* format) const
    {
#if __GNUC__ >= 3
       char temp[1024];
#endif
       s << "\n";
       for (int i = 0; i < rows_; i++)
       {
          for (int j = 0; j < cols_; j++)
          {
#if __GNUC__ < 3
             s << form(format, mat_[i][j]) << " ";
#else
             std::sprintf(temp, format, mat_[i][j]);
             s << temp << " ";
#endif
          }
          s << "\n";
       }
    }

    // ٥ȥ˽񤭹
    void  Mat::setcvec(int n, VecC& V)
    {
       assert((n >= 0) && (n < cols_));

       for (int i = 0; i < rows_; i++)
          mat_[i].vec_[n] = V[i];
    }

    // ޥȥꥯ黻(û)
    Mat&  Mat::operator += (const Mat& M)
    {
       assert(M.rows_ == rows_ && M.cols_ == cols_);
       for (int i = 0; i < rows_; i++)
          for (int j = 0; j < cols_; j++)
             mat_[i].vec_[j] += M[i].vec_[j];

       return *this;
    }

    // ޥȥꥯ黻()
    Mat&  Mat::operator -= (const Mat& M)
    {
       assert(M.rows_ == rows_ && M.cols_ == cols_);
       for (int i = 0; i < rows_; i++)
          for (int j = 0; j < cols_; j++)
             mat_[i].vec_[j] -= M[i].vec_[j];

       return *this;
    }

    // ޥȥꥯ黻(軻)
    Mat&  Mat::operator *= (const Mat& M)
    {
       int      i, j;
       Mat      T(rows_, cols_);

       assert(M.rows_ == rows_ && M.cols_ == cols_);

       for (i = 0; i < rows_; i++)
          for (j = 0; j < cols_; j++)
          {
             T[i].vec_[j] = 0.0;
             for (int k = 0; k < cols_; k++)
                T[i].vec_[j] += mat_[i].vec_[k] * M[k].vec_[j];
          }

       for (i = 0; i < rows_; i++)
          for (j = 0; j < cols_; j++)
             mat_[i].vec_[j] = T[i].vec_[j];

       return *this;
    }

    // ޥȥꥯ黻(Ȥξ軻)
    Mat&  Mat::operator *= (const double a)
    {
       for (int i = 0; i < rows_; i++)
          for (int j = 0; j < cols_; j++)
             mat_[i].vec_[j] *= a;

       return *this;
    }

    // ޥȥꥯ黻(Ȥν)
    Mat&  Mat::operator /= (const double a)
    {
       for (int i = 0; i < rows_; i++)
          for (int j = 0; j < cols_; j++)
             mat_[i].vec_[j] /= a;

       return *this;
    }

    // ޥȥꥯȤν
    Mat  operator / (const Mat& M, double a)
    {
       Mat  T(M);

       for (int i = 0; i < M.rows_; i++)
          for (int j = 0; j < M.cols_; j++)
             T.mat_[i][j] /= a;

       return T;
    }

    // ޥȥꥯ֤ξ軻
    Mat  operator * (const Mat& M1, const Mat& M2)
    {
       Mat  T(M1.rows_, M2.cols_);

       assert(M1.cols_ == M2.rows_);

       for (int i = 0; i < M1.rows_; i++)
          for (int j = 0; j < M2.cols_; j++)
          {
             T.mat_[i][j] = 0.0;
             for (int k = 0; k < M1.cols_; k++)
                T.mat_[i][j] += M1.mat_[i][k] * M2.mat_[k][j];
          }
       return T;
    }

    // ޥȥꥯȤξ軻
    Mat  operator * (const Mat& M, double a)
    {
       Mat  T(M);

       for (int i = 0; i < T.rows_; i++)
          for (int j = 0; j < T.cols_; j++)
             T.mat_[i][j] *= a;

       return T;
    }

    // ޥȥꥯȤξ軻(2)
    Mat  operator * (double a, const Mat& M)
    {
       return (M * a);
    }

    // ޥȥꥯβû
    Mat  operator + (const Mat& M1, const Mat& M2)
    {
       Mat  T(M1);

       assert(M1.cols_ == M2.cols_ && M1.rows_ == M2.rows_);
       for (int i = 0; i < M1.rows_; i++)
          for (int j = 0; j < M1.cols_; j++)
             T.mat_[i][j] += M2.mat_[i][j];

       return T;
    }

    // ޥȥꥯθ
    Mat  operator - (const Mat& M1, const Mat& M2)
    {
       Mat  T(M1);

       assert(M1.cols_ == M2.cols_ && M1.rows_ == M2.rows_);
       for (int i = 0; i < M1.rows_; i++)
          for (int j = 0; j < M1.cols_; j++)
             T.mat_[i][j] -= M2.mat_[i][j];

       return T;
    }

    // ñΥޥʥ
    Mat  operator - (const Mat& M)
    {
       Mat  T(M);

       for (int i = 0; i < M.rows_; i++)
          for (int j = 0; j < M.cols_; j++)
             T.mat_[i][j] = -T.mat_[i][j];

       return T;
    }

    // ޥȥꥯž
    Mat  operator ~ (const Mat& M)
    {
       Mat  T(M.cols_, M.rows_);

       for (int i = 0; i < T.rows_; i++)
          for (int j = 0; j < T.cols_; j++)
             T.mat_[i][j] = M.mat_[j][i];

       return T;
    }

    // ޥȥꥯ
    double  operator , (const Mat& M1, const Mat& M2)
    {
       double   d = 0.0;

       assert(M1.cols_ == M2.cols_ && M1.rows_ == M2.rows_);

       for (int i = 0; i < M1.rows_; i++)
          for (int j = 0; j < M1.cols_; j++)
             d += M1.mat_[i][j] * M2.mat_[i][j];

       return d;
    }

    // ޥȥꥯ()
    bool  operator == (const Mat& M1, const Mat& M2)
    {
       assert(M1.col() == M2.col() && M1.row() == M2.row());

       for (int i = 0; i < M1.row(); i++)
          for (int j = 0; j < M1.col(); j++)
             if (fabs(M1[i][j] - M2[i][j]) > EPS)
                return false;
       return true;
    }

    // ޥȥꥯ(԰)
    bool  operator != (const Mat& M1, const Mat& M2)
    {
       assert(M1.col() == M2.col() && M1.row() == M2.row());

       for (int i = 0; i < M1.row(); i++)
          for (int j = 0; j < M1.col(); j++)
             if (fabs(M1[i][j] - M2[i][j]) > EPS)
                return true;
       return false;
    }

    // ޥȥꥯ٥ȥξ軻  ٥ȥ
    VecC  operator * (const Mat& M, const VecC& V)
    {
       VecC T(M.row());

       assert(M.col() == V.size());

       for (int i = 0; i < M.row(); i++)
       {
          T[i] = 0.0;
          for (int j = 0; j < M.col(); j++)
             T[i] += M[i][j] * V[j];
       }

       return T;
    }

    // ԥ٥ȥȥޥȥꥯξ軻  ԥ٥ȥ
    VecR  operator * (const VecR& V, const Mat& M)
    {
       VecR T(M.col());

       assert(M.row() == V.size());

       for (int i = 0; i < M.col(); i++)
       {
          T[i] = 0.0;
          for (int j = 0; j < M.row(); j++)
             T[i] += M[j][i] * V[j];
       }

       return T;
    }

    // ٥ȥ * ԥ٥ȥ => ޥȥꥯ
    Mat  operator * (const VecC& CV, const VecR& RV)
    {
       Mat  M(CV.size(), RV.size());

       for (int i = 0; i < CV.size(); i++)
          for (int j = 0; j < RV.size(); j++)
             M[i][j] = CV[i] * RV[j];

       return M;
    }

    // ΥΥ
    double  Mat::norm() const
    {
       return sqrt(this->sqnorm());
    }

    // Υ
    double  Mat::sqnorm() const
    {
       double   d = 0.0;

       for (int i = 0; i < rows_; i++)
          for (int j = 0; j < cols_; j++)
             d += mat_[i].vec_[j] * mat_[i].vec_[j];

       return d;
    }


    // ϥڥ졼
    istream&  operator >> (istream& s, Mat& M)
    {
       for (int i = 0; i < M.rows_; i++)
          s >> M.mat_[i];

       return s;
    }

    // ϥڥ졼
    ostream&  operator << (ostream& s, const Mat& M)
    {
       s << "\n";
       for (int i = 0; i < M.rows_; i++)
          s << M.mat_[i] << "\n";

       return s;
    }

    // ðʬη̤򥽡
    void  svdsort(Mat& U, VecC& S, Mat& V)
    {
       Mat     U1(U.row(), U.col()), V1(V.row(), V.col());
       VecC    S1(S.size()), map(S.size());
       double  max;
       int     i, j, pos = 0, prepos;

       if ((U.col() != S.size()) || (S.size() != V.col()))
       {
          cerr << "Mat or Vec's size is not correct" << endl;
          exit(-1);
       }
       for (i = 0; i < S.size(); i++)
       {
          max = -1.0;
          for (j = 0; j < S.size(); j++)
             if (S[j] > max)
             {
                max = S[j];
                pos = j;
             }
          S1[i] = max;
          map[i] = pos;
          S[pos] = -1.0;
       }
       prepos = S.size();
       for (i = 0; i < S.size(); i++)
       {
          if (map[i] == prepos)
             break;
          for (j = 0; j < U.row(); j++)
             U1[j][i] = U[j][(int)map[i]];
          for (j = 0; j < V.row(); j++)
             V1[j][i] = V[j][(int)map[i]];
          prepos = (int)map[i];
       }
       U = U1;
       S = S1;
       V = V1;
    }

    // չ
    Mat  inverse(const Mat& M)
    {
       Mat      T(M);
       double   **a, **y;
       int      i, j, n;

       assert(M.col() == M.row());
       n = M.col();
       a = matrix(1, n, 1, n);  // recipes
       y = matrix(1, n, 1, n);  // recipes

       for (i = 0; i < n; i++)  // Ѥ˥ԡ
          for (j = 0; j < n; j++)
             a[i+1][j+1] = M[i][j];

       rc_inverse(a, y, n);     // 饤֥(librecipes.a)θƤӽФ

       for (i = 0; i < n; i++)  // Ϥ饳ԡ
          for (j = 0; j < n; j++)
             T[i][j] = y[i+1][j+1];

       free_matrix(a, 1, n, 1, n);  // recipes
       free_matrix(y, 1, n, 1, n);  // recipes

       return T;
    }

    // 33ι
    double  det33(const Mat& M)
    {
       double   d;

       assert(M.row() == 3 && M.col() == 3);

       d = M[0][0] * M[1][1] * M[2][2]
         + M[0][2] * M[2][1] * M[1][0]
         + M[0][1] * M[1][2] * M[2][0]
         - M[0][2] * M[1][1] * M[2][0]
         - M[0][1] * M[1][0] * M[2][2]
         - M[0][0] * M[2][1] * M[1][2];

       return d;
    }

    // ޥȥꥯ(EPS꾮Τ0)
    Mat  ignore_EPS(const Mat& M0)
    {
       Mat  M(M0.row(), M0.col());

       for (int i = 0; i < M0.row(); i++)
          for (int j = 0; j < M0.col(); j++)
             M[i][j] = (fabs(M0[i][j]) < DET_EPS)? 0.0: M0[i][j];

       return M;
    }

    // 33ι(EPS꾮Τ0Ȥ)
    double  det33EPS(const Mat& M0)
    {
       double   d;
       Mat      M;

       assert(M0.row() == 3 && M0.col() == 3);

       M = ignore_EPS(M0);

       d = M[0][0] * M[1][1] * M[2][2]
         + M[0][2] * M[2][1] * M[1][0]
         + M[0][1] * M[1][2] * M[2][0]
         - M[0][2] * M[1][1] * M[2][0]
         - M[0][1] * M[1][0] * M[2][2]
         - M[0][0] * M[2][1] * M[1][2];

       return d;
    }

    // ̤ι󼰤
    double  det(const Mat& M)
    {
        double  **a, d;

        assert(M.row() == M.col());
        int n = M.row();
        a = matrix(1, n, 1, n); // recipes

        for (int i = 0; i < n; i++) // Ѥ˥ԡ
            for (int j = 0; j < n; j++)
                a[i+1][j+1] = M[i][j];

        d = rc_det(a, n);       // 饤֥(librecipes.a)θƤӽФ

        free_matrix(a, 1, n, 1, n); // recipes
        return d;
    }

    // ͭͤȸͭ٥ȥ
    // EV: ٥ȥ뤬ͭ٥ȥ
    // V:  ͭ
    void  eigen_vector(const Mat& M, Mat& EV, VecC& V)
    {
        double  **m, *d, *e;
        bool    flg = false;
        double  diff_max = 0.0;

        assert(M.row() == M.col());
        assert(EV.row() == EV.col() && EV.row() == M.row());
        assert(M.col() == V.size());

        // оȽ
        for (int i = 0; i < M.row(); i++)
            for (int j = 0; j < M.col(); j++)
                if (i != j && fabs(M[i][j] - M[j][i]) > EPS)
                {
                    flg = true;
                    if (diff_max < fabs(M[i][j] - M[j][i]))
                        diff_max = fabs(M[i][j] - M[j][i]);
                }
                if (flg)
                {
                    cerr << "Warning : This Matrix is not real symmetric matrix"
                            " (in eigen_vector)." << endl;
#if __GNUC__ < 3
                    cerr << form("Max |M[i][j] - M[j][i]| = %10.5e", diff_max)
                         << endl;
#else
                    cerr << "Max |M[i][j] - M[j][i]| = "
                         << std::scientific << std::setprecision(5) << diff_max
                         << endl;
#endif
                }

        int n = M.row();
        m = matrix(1, n, 1, n); // recipes
        d = recipes_vector(1, n);       // recipes
        e = recipes_vector(1, n);       // recipes

        for (int i = 0; i < n; i++) // Ѥ˥ԡ
            for (int j = 0; j < n; j++)
                m[i+1][j+1] = M[i][j];

        tred2(m, n, d, e);      // HouseholderѴ
        tqli(d, e, n, m);       // ŪեȤĤQLˡ
        eigsrt(d, m, n);        // ͭͤӸͭ٥ȥΥ

        for (int i = 0; i < n; i++) // Ϥ饳ԡ
        {
            V[i] = d[i+1];
            for (int j = 0; j < n; j++)
                EV[i][j] = m[i+1][j+1];
        }

        free_matrix(m, 1, n, 1, n); // recipes
        free_vector(d, 1, n);   // recipes
        free_vector(e, 1, n);   // recipes
    }

    // ǾͭͤФͭ٥ȥ
    VecC  min_eigen_vector(const Mat& M)
    {
        Mat EgVec(M);
        VecC EgVal(M.col()), u(M.col());
        int k;

        eigen_vector(M, EgVec, EgVal);

        k = M.col() - 1;
        for (int i = 0; i < EgVec.row(); i++)
            u[i] = EgVec[i][k];

        return u;
    }

    // ǾͭͤФͭ٥ȥȤθͭ
    VecC  min_eigen_vector2(const Mat& M, double* l)
    {
        Mat EgVec(M);
        VecC EgVal(M.col()), u(M.col());
        int k;

        eigen_vector(M, EgVec, EgVal);

        k = M.col() - 1;
        for (int i = 0; i < EgVec.row(); i++)
            u[i] = EgVec[i][k];

        *l = EgVal[k];

        return u;
    }

    // 쥹ʬԤ
    Mat  CholeskyDecomp(const Mat& M)
    {
        assert(M.row() == M.col());
        Mat L(M.row(), M.col());
        L.clear();

        for (int i = 0; i < M.row(); ++i)
        {
            for (int j = 0; j < i; ++j)
            {
                for (int k = 0; k < j; ++k)
                    L[i][j] += L[i][k] * L[j][k];
                L[i][j] = (M[i][j] - L[i][j]) / L[j][j];
            }

            for (int k = 0; k < i; ++k)
                L[i][i] += L[i][k] * L[i][k];
            L[i][i] = std::sqrt(M[i][i] - L[i][i]);
        }

        return L;
    }

    // ðʬԤʤ
    void  SVD(const Mat& M, Mat& V, VecC& S, Mat& U)
    {
        double  **v, *s, **u;
        int     i, j, n, m;

        assert(M.col() == V.col());  // MVιԿƱ
        assert(M.row() == V.row());  // MVƱ
        assert(V.col() == S.size());
        assert(S.size() == U.col());  // U ž֤Ƴݤ뤫

        m = M.row();
        n = M.col();
        v = matrix(1, m, 1, n); // recipes
        s = recipes_vector(1, n);       // recipes
        u = matrix(1, n, 1, n); // recipes

        for (i = 0; i < m; i++) // Ѥ˥ԡ
            for (j = 0; j < n; j++)
                v[i+1][j+1] = M[i][j];

        svdcmp(v, m, n, s, u);  // ðʬ

        for (i = 0; i < m; i++) // ѤϤ˥ԡ
            for (j = 0; j < n; j++)
                V[i][j] = v[i+1][j+1];

        for (i = 0; i < n; i++)
        {
            S[i] = s[i+1];
            for (j = 0; j < n; j++)
                U[i][j] = u[i+1][j+1];
        }

        svdsort(V, S, U);           // ðʬη̤򥽡

        free_matrix(v, 1, m, 1, n); // recipes
        free_vector(s, 1, n);       // recipes
        free_matrix(u, 1, n, 1, n); // recipes
    }

    // LU ʬˤϢΩ켡β
    VecC  SLE(const Mat& A, const VecC& B)
    {
        VecC    X(B.size());
        double  **a, *b, d;
        int     i, j, n, *indx;

        assert(A.col() == A.row());
        assert(A.col() == B.size());

        n = A.col();

        a = matrix(1, n, 1, n);
        b = recipes_vector(1, n);
        indx = ivector(1, n);

        for (i = 0; i < n; i++)
        {
            b[i+1] = B[i];
            for (j = 0; j < n; j++)
                a[i+1][j+1] = A[i][j];
        }

        ludcmp(a, n, indx, &d);
        lubksb(a, n, indx, b);

        for (i = 0; i < n; i++)
            X[i] = b[i+1];

        free_matrix(a, 1, n, 1, n);
        free_vector(b, 1, n);
        free_ivector(indx, 1, n);

        return X;
    }

    // ̵չ
    Mat  gen_inv(const Mat& M)
    {
        Mat T(M.col(), M.row());
        double  **a, **x;
        int     i, j, m, n;

        m = M.row();
        n = M.col();
        a = matrix(1, m, 1, n); // recipes
        x = matrix(1, n, 1, m); // recipes

        for (i = 0; i < m; i++) // Ѥ˥ԡ
            for (j = 0; j < n; j++)
                a[i+1][j+1] = M[i][j];

        rc_gen_inv(a, x, m, n); // 饤֥(librecipes.a)

        for (i = 0; i < n; i++) // Ϥ饳ԡ
            for (j = 0; j < m; j++)
                T[i][j] = x[i+1][j+1];

        free_matrix(a, 1, m, 1, n); // recipes
        free_matrix(x, 1, n, 1, m); // recipes

        return T;
    }
    // ݤ̵չ
    Mat  trc_gen_inv(const Mat& M, int r)
    {
        Mat T(M.col(), M.row());
        double  **a, **x;
        int     i, j, m, n;

        m = M.row();
        n = M.col();
        a = matrix(1, m, 1, n); // recipes
        x = matrix(1, n, 1, m); // recipes

        for (i = 0; i < m; i++) // Ѥ˥ԡ
            for (j = 0; j < n; j++)
                a[i+1][j+1] = M[i][j];

        rc_trunc_gen_inv(a, x, m, n, r); // 饤֥(librecipes.a)

        for (i = 0; i < n; i++) // Ϥ饳ԡ
            for (j = 0; j < m; j++)
                T[i][j] = x[i+1][j+1];

        free_matrix(a, 1, m, 1, n); // recipes
        free_matrix(x, 1, n, 1, m); // recipes

        return T;
    }

    // nnñ̹
    Mat  id_matrix(int n = 3)
    {
        Mat T(n, n);

        T.clear(0.0);
        for (int i = 0; i < n; i++)
            T[i][i] = 1.0;

        return T;
    }

    // оβ
    Mat  sym(const Mat& M)
    {
        return (M + (~M)) / 2.0;
    }

    // 3٥ȥ33Ȥγ
    Mat  operator % (const VecC& v, const Mat& M)
    {
        VecC x0, x1, x2;
        Mat T(3, 3);

        assert(M.col() == M.row() && M.col() == 3);
        assert(v.size() == 3);

        x0 = v % M(0);
        x1 = v % M(1);
        x2 = v % M(2);
        T.setcvec(0, x0);
        T.setcvec(1, x1);
        T.setcvec(2, x2);

        return T;
    }

    // 333٥ȥȤγ
    Mat  operator % (const Mat& M, const VecC& v)
    {
        Mat I = id_matrix(3);
        return (M * (~(v % I)));
    }

    // 33γ(exterior product)
    Mat  ext_product(const Mat& A, const Mat& B)
    {
        Mat T(3,3);

        assert(A.col() == A.row() && A.col() == 3);
        assert(B.col() == B.row() && B.col() == 3);

        T[0][0] = + A[1][1] * B[2][2] - A[1][2] * B[2][1]
                  - A[2][1] * B[1][2] + A[2][2] * B[1][1];
        T[0][1] = - A[1][0] * B[2][2] + A[1][2] * B[2][0]
                  + A[2][0] * B[1][2] - A[2][2] * B[1][0];
        T[0][2] = + A[1][0] * B[2][1] - A[1][1] * B[2][0]
                  - A[2][0] * B[1][1] + A[2][1] * B[1][0];
        T[1][0] = - A[0][1] * B[2][2] + A[0][2] * B[2][1]
                  + A[2][1] * B[0][2] - A[2][2] * B[0][1];
        T[1][1] = + A[0][0] * B[2][2] - A[0][2] * B[2][0]
                  - A[2][0] * B[0][2] + A[2][2] * B[0][0];
        T[1][2] = - A[0][0] * B[2][1] + A[0][1] * B[2][0]
                  + A[2][0] * B[0][1] - A[2][1] * B[0][0];
        T[2][0] = + A[0][1] * B[1][2] - A[0][2] * B[1][1]
                  - A[1][1] * B[0][2] + A[1][2] * B[0][1];
        T[2][1] = - A[0][0] * B[1][2] + A[0][2] * B[1][0]
                  + A[1][0] * B[0][2] - A[1][2] * B[0][0];
        T[2][2] = + A[0][0] * B[1][1] - A[0][1] * B[1][0]
                  - A[1][0] * B[0][1] + A[1][1] * B[0][0];

        return T;
    }

    // 33;ҹ
    Mat  cofactor33(const Mat& M)
    {
        Mat T(3,3);

        T[0][0] = M[1][1] * M[2][2] - M[2][1] * M[1][2];
        T[0][1] = M[2][1] * M[0][2] - M[0][1] * M[2][2];
        T[0][2] = M[0][1] * M[1][2] - M[1][1] * M[0][2];
        T[1][0] = M[1][2] * M[2][0] - M[2][2] * M[1][0];
        T[1][1] = M[2][2] * M[0][0] - M[0][2] * M[2][0];
        T[1][2] = M[0][2] * M[1][0] - M[1][2] * M[0][0];
        T[2][0] = M[1][0] * M[2][1] - M[2][0] * M[1][1];
        T[2][1] = M[2][0] * M[0][1] - M[0][0] * M[2][1];
        T[2][2] = M[0][0] * M[1][1] - M[1][0] * M[0][1];

        return T;
    }

    // 33ޥȥꥯ3Ĥ٥ȥʬ
    void  col3(const Mat& M, VecC& u1, VecC& u2, VecC& u3)
    {
        assert(M.row() == 3 && M.col() == 3);
        assert(u1.size() == 3 && u2.size() == 3 && u3.size() == 3);

        for (int i = 0; i < 3; i++)
        {
            u1[i] = M[i][0];
            u2[i] = M[i][1];
            u3[i] = M[i][2];
        }
    }

    // (ե٥˥Υ1ˤ)
    Mat  normalize(const Mat& M)
    {
        Mat     T(M);
        double  d;

        if ((d = T.norm()) < EPS)
        {
            T.clear(0.0);
            return T;
        }
        return T / d;
    }

    // Υȥ졼
    double  trace(const Mat& M)
    {
        double  t;

        assert(M.col() == M.row());

        t = 0.0;
        for (int i = 0; i < M.row(); i++)
            t += M[i][i];

        return t;
    }

    // Ϳ줿ͤгǤȤ33
    Mat  diag33(double a, double b, double c)
    {
        Mat T;
        T.clear();

        T[0][0] = a;
        T[1][1] = b;
        T[2][2] = c;

        return T;
    }

    // ٥ȥǤгǤȤ
    Mat  diag(const Vec& V)
    {
        Mat T(V.size(), V.size());
        T.clear();

        for(int i = 0; i < V.size(); i++)
            T[i][i] = V[i];

        return T;
    }

    // Ϳ줿гǤΤߤι
    Mat  diag(const Mat& M)
    {
       Mat  T(M.row(), M.col());
       T.clear();

       for(int i = 0; i < M.row(); i++)
          T[i][i] = M[i][i];

       return T;
    }
}
